local addon = LibStub('AceAddon-3.0'):GetAddon('KuiNameplates')
local mod = addon:NewModule('ComboPoints', 'AceEvent-3.0')

mod.uiName = 'Combo points'

local function ComboPointsUpdate(self)
	if self.points and self.points > 0 then
		local i
		
		for i = 1,5 do
			if i <= self.points then
				self[i]:SetAlpha(1)
			else
				self[i]:SetAlpha(.3)
			end

			self[i]:SetVertexColor(unpack(self.colour))
		end

		self:Show()
	elseif self:IsShown() then
		self:Hide()
	end
end
-------------------------------------------------------------- Event handlers --
function mod:UNIT_COMBO_POINTS(event, unit)
	-- only works for player > target
	if not unit or unit ~= 'player' then return end

	-- TODO make sure UnitName() doesn't need to have realm names removed
	-- before GetNameplate will recognise them
	local guid, name = UnitGUID('target'), UnitName('target')
	local f = addon:GetNameplate(guid, name)
	
	if f and f.combopoints then
		local points = GetComboPoints('player', 'target')

		if points == 5 then
			f.combopoints.colour = { 1, 1, .1 }
		else
			f.combopoints.colour = { .79, .55, .18 }
		end

		f.combopoints.points = points
		f.combopoints:Update()
	end

	-- clear points on other frames
	local _, frame
	for _, frame in pairs(addon.frameList) do
		if frame.combopoints and frame ~= f then
			self:HideComboPoints(nil, frame)
		end
	end
end
---------------------------------------------------------------------- Create --
function mod:CreateComboPoints(msg, frame)
	-- create combo point icons
	frame.combopoints = CreateFrame('Frame', nil, frame.parent)
	frame.combopoints:Hide()

	local i, pcp
	for i=0,4 do
		local cp = frame.combopoints:CreateTexture(nil, 'ARTWORK')
		cp:SetDrawLayer('ARTWORK', 1) -- above overlay
		cp:SetTexture('Interface\\AddOns\\Kui_Nameplates\\media\\combopoint')
		cp:SetTexCoord(0, .375, 0, .375)
		cp:SetSize(addon.sizes.combopoints, addon.sizes.combopoints)

		if i == 0 then
			cp:SetPoint('BOTTOM', frame.overlay, 'BOTTOM',
				-((addon.sizes.combopoints*2)+2), -3)
		else
			cp:SetPoint('LEFT', pcp, 'RIGHT', 1, 0)
		end

		tinsert(frame.combopoints, i+1, cp)
		pcp = cp -- store previous icon
	end

	frame.combopoints.Update = ComboPointsUpdate
end
------------------------------------------------------------------------ Hide --
function mod:HideComboPoints(msg, frame)
	if frame.combopoints then
		frame.combopoints.points = nil
		frame.combopoints:Update()
	end
end
---------------------------------------------------- Post db change functions --
mod.configChangedFuncs = { runOnce = {} }
mod.configChangedFuncs.runOnce.enabled = function(val)
	if val then
		mod:Enable()
	else
		mod:Disable()
	end
end

-------------------------------------------------------------------- Register --
function mod:GetOptions()
	return {
		enabled = {
			name = 'Show combo points',
			desc = 'Show combo points on the target',
			type = 'toggle'
		},
	}
end

function mod:OnInitialize()
	self.db = addon.db:RegisterNamespace(self.moduleName, {
		profile = {
			enabled = true,
		}
	})

	addon:RegisterSize('frame', 'combopoints', 5)
	addon:InitModuleOptions(self)
	mod:SetEnabledState(self.db.profile.enabled)
end

function mod:OnEnable()
	self:RegisterMessage('KuiNameplates_PostCreate', 'CreateComboPoints')
	self:RegisterMessage('KuiNameplates_PostHide', 'HideComboPoints')

	self:RegisterEvent('UNIT_COMBO_POINTS')

	local _, frame
	for _, frame in pairs(addon.frameList) do
		if not frame.combopoints then
			self:CreateComboPoints(nil, frame.kui)
		end
	end
end

function mod:OnDisable()
	self:UnregisterEvent('UNIT_COMBO_POINTS')

	local _, frame
	for _, frame in pairs(addon.frameList) do
		self:HideComboPoints(nil, frame.kui)
	end
end
